@@ -16,10 +16,11 @@ from . import sample
 from . import caching
 from . import inertia
 from . import creation
-from . import transformations
+from . import triangles
+from . import transformations as tf
 
 from .base import Trimesh
-from .constants import log
+from .constants import log, tol
 
 
 class _Primitive(Trimesh):
@@ -28,11 +29,18 @@ class _Primitive(Trimesh):
     Mesh is generated lazily when vertices or faces are requested.
     """
 
+    # ignore superclass copy directives
+    __copy__ = None
+    __deepcopy__ = None
+
     def __init__(self, *args, **kwargs):
         super(_Primitive, self).__init__(*args, **kwargs)
         self._data.clear()
         self._validate = False
 
+    def __repr__(self):
+        return '<trimesh.primitives.{}>'.format(type(self).__name__)
+
     @property
     def faces(self):
         stored = self._cache['faces']
@@ -61,24 +69,34 @@ class _Primitive(Trimesh):
 
     @property
     def face_normals(self):
+        # we need to avoid the logic in the superclass that
+        # is specific to the data model prioritizing faces
         stored = self._cache['face_normals']
         if util.is_shape(stored, (-1, 3)):
             return stored
-        self._create_mesh()
-        return self._cache['face_normals']
+        # just calculate if not stored
+        unit, valid = triangles.normals(self.triangles)
+        normals = np.zeros((len(valid), 3))
+        normals[valid] = unit
+        # store and return
+        self._cache['face_normals'] = normals
+        return normals
 
     @face_normals.setter
     def face_normals(self, values):
         if values is not None:
             log.warning('Primitive face normals are immutable! Not setting!')
 
-    def copy(self):
+    def copy(self, **kwargs):
         """
         Return a copy of the Primitive object.
+
+        Returns
+        -------------
+        copied : object
+          Copy of current primitive
         """
-        result = copy.deepcopy(self)
-        result._cache.clear()
-        return result
+        return copy.deepcopy(self)
 
     def to_mesh(self):
         """
@@ -92,23 +110,25 @@ class _Primitive(Trimesh):
 
     def apply_transform(self, matrix):
         """
-        Apply a transform to the current primitive (sets self.transform)
+        Apply a transform to the current primitive by
+        setting self.transform
 
         Parameters
-        -----------
-        matrix: (4,4) float, homogenous transformation
+        ------------
+        matrix: (4,4) float
+          Homogeneous transformation
         """
         matrix = np.asanyarray(matrix, order='C', dtype=np.float64)
         if matrix.shape != (4, 4):
             raise ValueError('Transformation matrix must be (4,4)!')
 
-        if np.allclose(matrix, np.eye(4)):
-            log.debug('apply_tranform received identity matrix')
+        if util.allclose(matrix, np.eye(4), 1e-8):
+            log.debug('apply_transform received identity matrix')
             return
 
         new_transform = np.dot(matrix, self.primitive.transform)
-
         self.primitive.transform = new_transform
+        return self
 
     def _create_mesh(self):
         raise ValueError('Primitive doesn\'t define mesh creation!')
@@ -127,7 +147,8 @@ class _PrimitiveAttributes(object):
         self._mutable = True
         for key, value in kwargs.items():
             if key in defaults:
-                self._data[key] = util.convert_like(value, defaults[key])
+                self._data[key] = util.convert_like(
+                    value, defaults[key])
         # if configured as immutable, apply setting after instantiation values
         # are set
         if 'mutable' in kwargs:
@@ -176,6 +197,19 @@ class _PrimitiveAttributes(object):
             raise ValueError(
                 'Only default attributes {} can be set!'.format(keys))
 
+    def to_kwargs(self):
+        """
+        Return a dict with copies of kwargs for the current
+        Primitive.
+
+        Returns
+        ------------
+        kwargs : dict
+          Arguments to reconstruct current PrimitiveAttributes
+        """
+        return {k: copy.deepcopy(self._data[k])
+                for k in self._defaults.keys()}
+
     def __dir__(self):
         result = sorted(dir(type(self)) +
                         list(self._defaults.keys()))
@@ -189,11 +223,15 @@ class Cylinder(_Primitive):
         Create a Cylinder Primitive, a subclass of Trimesh.
 
         Parameters
-        ----------
-        radius: float, radius of cylinder
-        height: float, height of cylinder
-        transform: (4,4) float, transformation matrix
-        sections: int, number of facets in circle
+        -------------
+        radius : float
+          Radius of cylinder
+        height : float
+          Height of cylinder
+        transform : (4, 4) float
+          Homogeneous transformation matrix
+        sections : int
+          Number of facets in circle
         """
         super(Cylinder, self).__init__(*args, **kwargs)
 
@@ -212,9 +250,11 @@ class Cylinder(_Primitive):
 
         Returns
         ---------
-        volume: float, volume of the cylinder
+        volume : float
+          Volume of the cylinder
         """
-        volume = (np.pi * self.primitive.radius ** 2) * self.primitive.height
+        volume = ((np.pi * self.primitive.radius ** 2) *
+                  self.primitive.height)
         return volume
 
     @caching.cache_decorator
@@ -227,13 +267,14 @@ class Cylinder(_Primitive):
         tensor: (3,3) float, 3D inertia tensor
         """
 
-        tensor = inertia.cylinder_inertia(mass=self.volume,
-                                          radius=self.primitive.radius,
-                                          height=self.primitive.height,
-                                          transform=self.primitive.transform)
+        tensor = inertia.cylinder_inertia(
+            mass=self.volume,
+            radius=self.primitive.radius,
+            height=self.primitive.height,
+            transform=self.primitive.transform)
         return tensor
 
-    @property
+    @caching.cache_decorator
     def direction(self):
         """
         The direction of the cylinder's axis.
@@ -245,12 +286,51 @@ class Cylinder(_Primitive):
         axis = np.dot(self.primitive.transform, [0, 0, 1, 0])[:3]
         return axis
 
-    def _create_mesh(self):
-        log.info('Creating cylinder mesh with r=%f, h=%f and %d sections',
-                 self.primitive.radius,
-                 self.primitive.height,
-                 self.primitive.sections)
+    @property
+    def segment(self):
+        """
+        A line segment which if inflated by cylinder radius
+        would represent the cylinder primitive.
 
+        Returns
+        -------------
+        segment : (2, 3) float
+          Points representing a single line segment
+        """
+        # half the height
+        half = self.primitive.height / 2.0
+        # apply the transform to the Z- aligned segment
+        points = np.dot(
+            self.primitive.transform,
+            np.transpose([[0, 0, -half, 1], [0, 0, half, 1]])).T[:, :3]
+        return points
+
+    def buffer(self, distance):
+        """
+        Return a cylinder primitive which covers the source cylinder
+        by distance: radius is inflated by distance, height by twice
+        the distance.
+
+        Parameters
+        ------------
+        distance : float
+          Distance to inflate cylinder radius and height
+
+        Returns
+        -------------
+        buffered : Cylinder
+         Cylinder primitive inflated by distance
+        """
+        distance = float(distance)
+
+        buffered = Cylinder(
+            height=self.primitive.height + distance * 2,
+            radius=self.primitive.radius + distance,
+            transform=self.primitive.transform.copy())
+        return buffered
+
+    def _create_mesh(self):
+        log.debug('creating mesh for Cylinder primitive')
         mesh = creation.cylinder(radius=self.primitive.radius,
                                  height=self.primitive.height,
                                  sections=self.primitive.sections,
@@ -284,7 +364,7 @@ class Capsule(_Primitive):
                                               defaults,
                                               kwargs)
 
-    @property
+    @caching.cache_decorator
     def direction(self):
         """
         The direction of the capsule's axis.
@@ -297,10 +377,7 @@ class Capsule(_Primitive):
         return axis
 
     def _create_mesh(self):
-        log.info('Creating capsule mesh with r=%f, h=%f and %d sections',
-                 self.primitive.radius,
-                 self.primitive.height,
-                 self.primitive.sections)
+        log.debug('creating mesh for Capsule primitive')
 
         mesh = creation.capsule(radius=self.primitive.radius,
                                 height=self.primitive.height)
@@ -340,7 +417,7 @@ class Sphere(_Primitive):
 
         Parameters
         ------------
-        matrix: (4,4) float, homogenous transformation
+        matrix: (4,4) float, homogeneous transformation
         """
         matrix = np.asanyarray(matrix, dtype=np.float64)
         if matrix.shape != (4, 4):
@@ -365,7 +442,6 @@ class Sphere(_Primitive):
         # bounding box, and a sphere is the absolute slowest case for the OBB calculation
         # as it is a convex surface with a ton of face normals that all need to
         # be checked
-
         return self.bounding_box
 
     @caching.cache_decorator
@@ -408,6 +484,7 @@ class Sphere(_Primitive):
         return tensor
 
     def _create_mesh(self):
+        log.debug('creating mesh for Sphere primitive')
         unit = creation.icosphere(subdivisions=self.primitive.subdivisions)
         unit.vertices *= self.primitive.radius
         unit.vertices += self.primitive.center
@@ -442,15 +519,18 @@ class Box(_Primitive):
 
         Parameters
         -------------
-        count: int, number of samples to return
+        count : int
+          Number of samples to return
 
         Returns
         ----------
-        samples: (count,3) float, points inside the volume
+        samples : (count, 3) float
+          Points inside the volume
         """
-        samples = sample.volume_rectangular(extents=self.primitive.extents,
-                                            count=count,
-                                            transform=self.primitive.transform)
+        samples = sample.volume_rectangular(
+            extents=self.primitive.extents,
+            count=count,
+            transform=self.primitive.transform)
         return samples
 
     def sample_grid(self, count=None, step=None):
@@ -461,12 +541,15 @@ class Box(_Primitive):
 
         Parameters
         -----------
-        count: int   or (3,) int,   if specified samples are spaced with np.linspace
-        step:  float or (3,) float, if specified samples are spaced with np.arange
+        count : int or (3,) int
+          If specified samples are spaced with np.linspace
+        step : float or (3,) float
+          If specified samples are spaced with np.arange
 
         Returns
         -----------
-        grid: (n,3) float, points inside the box
+        grid : (n, 3) float
+          Points inside the box
         """
 
         if (count is not None and
@@ -484,7 +567,7 @@ class Box(_Primitive):
         else:
             raise ValueError('either count or step must be specified!')
 
-        transformed = transformations.transform_points(
+        transformed = tf.transform_points(
             grid, matrix=self.primitive.transform)
         return transformed
 
@@ -512,7 +595,7 @@ class Box(_Primitive):
         return volume
 
     def _create_mesh(self):
-        log.debug('Creating mesh for box Primitive')
+        log.debug('creating mesh for Box primitive')
         box = creation.box(extents=self.primitive.extents,
                            transform=self.primitive.transform)
 
@@ -523,25 +606,31 @@ class Box(_Primitive):
 
 class Extrusion(_Primitive):
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, triangle_args=None, *args, **kwargs):
         """
-        Create an Extrusion Primitive, a subclass of Trimesh
+        Create an Extrusion primitive, which
+        is a subclass of Trimesh.
 
         Parameters
         ----------
-        polygon:   shapely.geometry.Polygon, polygon to extrude
-        transform: (4,4) float, transform to apply after extrusion
-        height:    float, height to extrude polygon by
+        polygon : shapely.geometry.Polygon
+          Polygon to extrude
+        transform : (4,4) float
+          Transform to apply after extrusion
+        height : float
+          Height to extrude polygon by
+        triangle_args : str
+          Arguments to pass to triangle
         """
-        super(Extrusion, self).__init__(*args, **kwargs)
-
         # do the import here, fail early if Shapely isn't installed
         from shapely.geometry import Point
-
+        super(Extrusion, self).__init__(*args, **kwargs)
+        # save arguments for triangulation
+        self.triangle_args = triangle_args
+        # set default values
         defaults = {'polygon': Point([0, 0]).buffer(1.0),
                     'transform': np.eye(4),
                     'height': 1.0}
-
         self.primitive = _PrimitiveAttributes(self,
                                               defaults,
                                               kwargs)
@@ -555,7 +644,8 @@ class Extrusion(_Primitive):
 
         Returns
         ----------
-        area: float, surface area of 3D extrusion
+        area: float
+          Surface area of 3D extrusion
         """
         # area of the sides of the extrusion
         area = abs(self.primitive.height *
@@ -567,33 +657,69 @@ class Extrusion(_Primitive):
     @caching.cache_decorator
     def volume(self):
         """
-        The volume of the primitive extrusion.
-
+        The volume of the Extrusion primitive.
         Calculated from polygon and height to avoid mesh creation.
 
         Returns
         ----------
-        volume: float, volume of 3D extrusion
+        volume : float
+          Volume of 3D extrusion
         """
+        # height may be negative
         volume = abs(self.primitive.polygon.area *
                      self.primitive.height)
         return volume
 
-    @property
+    @caching.cache_decorator
     def direction(self):
         """
-        Based on the extrudes transform, what is the vector along
-        which the polygon will be extruded
+        Based on the extrudes transform what is the
+        vector along which the polygon will be extruded.
 
         Returns
         ---------
-        direction: (3,) float vector. If self.primitive.transform is an
-                   identity matrix this will be [0.0, 0.0, 1.0]
+        direction : (3,) float
+          Unit direction vector
         """
-        direction = np.dot(self.primitive.transform[:3, :3],
-                           [0.0, 0.0, np.sign(self.primitive.height)])
+        # only consider rotation and signed height
+        direction = np.dot(
+            self.primitive.transform[:3, :3],
+            [0.0, 0.0, np.sign(self.primitive.height)])
         return direction
 
+    @property
+    def origin(self):
+        """
+        Based on the extrude transform what is the
+        origin of the plane it is extruded from.
+
+        Returns
+        -----------
+        origin : (3,) float
+          Origin of extrusion plane
+        """
+        return self.primitive.transform[:3, 3]
+
+    @caching.cache_decorator
+    def bounding_box_oriented(self):
+        # no docstring for inheritance
+        # calculate OBB using 2D polygon and known axis
+        from . import bounds
+        # find the 2D bounding box using the polygon
+        to_origin, box = bounds.oriented_bounds_2D(
+            self.primitive.polygon.exterior.coords)
+        #  3D extents
+        extents = np.append(box, abs(self.primitive.height))
+        # calculate to_3D transform from 2D obb
+        rotation_Z = np.linalg.inv(tf.planar_matrix_to_3D(to_origin))
+        rotation_Z[2, 3] = self.primitive.height / 2.0
+        # combine the 2D OBB transformation with the 2D projection transform
+        to_3D = np.dot(self.primitive.transform, rotation_Z)
+        obb = Box(transform=to_3D,
+                  extents=extents,
+                  mutable=False)
+        return obb
+
     def slide(self, distance):
         """
         Alter the transform of the current extrusion to slide it
@@ -601,7 +727,8 @@ class Extrusion(_Primitive):
 
         Parameters
         -----------
-        distance: float, distance along self.extrude_direction to move
+        distance : float
+          Distance along self.extrude_direction to move
         """
         distance = float(distance)
         translation = np.eye(4)
@@ -610,38 +737,61 @@ class Extrusion(_Primitive):
                                translation.copy())
         self.primitive.transform = new_transform
 
-    def buffer(self, distance):
+    def buffer(self, distance, distance_height=None, **kwargs):
         """
-        Return a new Extrusion object which is expanded in profile and
-        in height by a specified distance.
+        Return a new Extrusion object which is expanded in profile
+        and in height by a specified distance.
+
+        Parameters
+        --------------
+        distance : float
+          Distance to buffer polygon
+        distance_height : float
+          Distance to buffer above and below extrusion
+        kwargs : dict
+          Passed to Extrusion constructor
 
         Returns
         ----------
-        buffered: Extrusion object
+        buffered : primitives.Extrusion
+          Extrusion object with new values
         """
         distance = float(distance)
+        # if not specified use same distance for everything
+        if distance_height is None:
+            distance_height = distance
 
         # start with current height
         height = self.primitive.height
         # if current height is negative offset by negative amount
-        height += np.sign(height) * 2.0 * distance
+        height += np.sign(height) * 2.0 * distance_height
 
-        buffered = Extrusion(
+        # create a new extrusion with a buffered polygon
+        # use type(self) vs Extrusion to handle subclasses
+        buffered = type(self)(
             transform=self.primitive.transform.copy(),
             polygon=self.primitive.polygon.buffer(distance),
-            height=height)
+            height=height,
+            **kwargs)
 
         # slide the stock along the axis
-        buffered.slide(-np.sign(height) * distance)
+        buffered.slide(-np.sign(height) * distance_height)
 
         return buffered
 
     def _create_mesh(self):
-        log.debug('Creating mesh for extrude Primitive')
-        mesh = creation.extrude_polygon(self.primitive.polygon,
-                                        self.primitive.height)
-        mesh.apply_transform(self.primitive.transform)
+        log.debug('creating mesh for Extrusion primitive')
+        # extrude the polygon along Z
+        mesh = creation.extrude_polygon(
+            polygon=self.primitive.polygon,
+            height=self.primitive.height,
+            transform=self.primitive.transform,
+            triangle_args=self.triangle_args)
 
+        # check volume here in unit tests
+        if tol.strict and mesh.volume < 0.0:
+            raise ValueError('matrix inverted mesh!')
+
+        # cache mesh geometry in the primitive
         self._cache['vertices'] = mesh.vertices
         self._cache['faces'] = mesh.faces
-        self._cache['face_normals'] = mesh.face_normals
